;*******************************************************************************
;*
;*      Titel:	Programm zum Aendern der I2C Adresse des CO2 Sensors K30
;*              (co22_K30_change_I2C_address.asm)
;*	Datum:	01/11/12			Version:0.2
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*      
;*	Prozessor:   ATmega32u4	      Quarzfrequenz: 16 MHz
;*      
;*	Eingaenge: 
;*	Ausgaenge: 
;*                  
;*	Informationen zur Funktionsweise: 
;*	
;*	SCL und SDA mit Pull-Up (1,2k) gegen 3V (DVCC) des Sensors
;*	Serielle Schnittstelle (9600, 8N1) an PD2 zur Kontrolle
;*	Ausgabe der alten Adresse und dann der neuen Adresse
;*	
;*      Copyright (c) 2012   Guy WEILER         weigu[at]weigu[dot]lu
;*      
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*      
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************

;-------------------------------------------------------------------------------
;	Einbinden der controllerspezifischen Definitionsdatei
;-------------------------------------------------------------------------------
.NOLIST				;List-Output ausschalten
.INCLUDE "m32U4def.inc"         ;AVR-Definitionsdatei einbinden
;folgende Zeilen fehlen in der Defenitionsdatei!
;Bugreport wurde von ATMEL bestaetigt
.equ    TWAMR   = 0xbd  ; MEMORY MAPPED
.equ    TWCR    = 0xbc  ; MEMORY MAPPED
.equ    TWDR    = 0xbb  ; MEMORY MAPPED
.equ    TWAR    = 0xba  ; MEMORY MAPPED
.equ    TWSR    = 0xb9  ; MEMORY MAPPED
.equ    TWBR    = 0xb8  ; MEMORY MAPPED
; ***** TWI **************************
; TWAMR - TWI (Slave) Address Mask Register
.equ    TWAM0   = 1     ;
.equ    TWAMR0  = TWAM0 ; For compatibility
.equ    TWAM1   = 2     ;
.equ    TWAMR1  = TWAM1 ; For compatibility
.equ    TWAM2   = 3     ;
.equ    TWAMR2  = TWAM2 ; For compatibility
.equ    TWAM3   = 4     ;
.equ    TWAMR3  = TWAM3 ; For compatibility
.equ    TWAM4   = 5     ;
.equ    TWAMR4  = TWAM4 ; For compatibility
.equ    TWAM5   = 6     ;
.equ    TWAMR5  = TWAM5 ; For compatibility
.equ    TWAM6   = 7     ;
.equ    TWAMR6  = TWAM6 ; For compatibility

; TWBR - TWI Bit Rate register
.equ    TWBR0   = 0     ;
.equ    TWBR1   = 1     ;
.equ    TWBR2   = 2     ;
.equ    TWBR3   = 3     ;
.equ    TWBR4   = 4     ;
.equ    TWBR5   = 5     ;
.equ    TWBR6   = 6     ;
.equ    TWBR7   = 7     ;

; TWCR - TWI Control Register
.equ    TWIE    = 0     ; TWI Interrupt Enable
.equ    TWEN    = 2     ; TWI Enable Bit
.equ    TWWC    = 3     ; TWI Write Collition Flag
.equ    TWSTO   = 4     ; TWI Stop Condition Bit
.equ    TWSTA   = 5     ; TWI Start Condition Bit
.equ    TWEA    = 6     ; TWI Enable Acknowledge Bit
.equ    TWINT   = 7     ; TWI Interrupt Flag

; TWSR - TWI Status Register
.equ    TWPS0   = 0     ; TWI Prescaler
.equ    TWPS1   = 1     ; TWI Prescaler
.equ    TWS3    = 3     ; TWI Status
.equ    TWS4    = 4     ; TWI Status
.equ    TWS5    = 5     ; TWI Status
.equ    TWS6    = 6     ; TWI Status
.equ    TWS7    = 7     ; TWI Status

; TWDR - TWI Data register
.equ    TWD0    = 0     ; TWI Data Register Bit 0
.equ    TWD1    = 1     ; TWI Data Register Bit 1
.equ    TWD2    = 2     ; TWI Data Register Bit 2
.equ    TWD3    = 3     ; TWI Data Register Bit 3
.equ    TWD4    = 4     ; TWI Data Register Bit 4
.equ    TWD5    = 5     ; TWI Data Register Bit 5
.equ    TWD6    = 6     ; TWI Data Register Bit 6
.equ    TWD7    = 7     ; TWI Data Register Bit 7

; TWAR - TWI (Slave) Address register
.equ    TWGCE   = 0     ; TWI General Call Recognition Enable Bit
.equ    TWA0    = 1     ; TWI (Slave) Address register Bit 0
.equ    TWA1    = 2     ; TWI (Slave) Address register Bit 1
.equ    TWA2    = 3     ; TWI (Slave) Address register Bit 2
.equ    TWA3    = 4     ; TWI (Slave) Address register Bit 3
.equ    TWA4    = 5     ; TWI (Slave) Address register Bit 4
.equ    TWA5    = 6     ; TWI (Slave) Address register Bit 5
.equ    TWA6    = 7     ; TWI (Slave) Address register Bit 6



.LIST			        ;List-Output wieder einschalten

;-------------------------------------------------------------------------------
;       Organisation des Datenspeichers (SRAM)
;-------------------------------------------------------------------------------
;.DSEG                           ;was ab hier folgt kommt in den SRAM-Speicher
;.ORG    SRAM_START


;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Programmspeicher (FLASH)     Programmstart nach RESET ab Adr. 0x0000
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
.ORG	0x0000			;Programm beginnt an der FLASH-Adresse 0x0000
RESET1:	rjmp	INIT		;springe nach INIT (ueberspringe ISR Vektoren)

;-------------------------------------------------------------------------------
;	Sprungadressen fuer die Interrupts organisieren (ISR VECTORS)
;-------------------------------------------------------------------------------
        ;Vektortabelle (im Flash-Speicher)

;-------------------------------------------------------------------------------
;	Initialisierungen und eigene Definitionen
;-------------------------------------------------------------------------------
.ORG    INT_VECTORS_SIZE        ;Platz fuer ISR Vektoren lassen
INIT:
.DEF    Zero = r15              ;Register 1 wird zum Rechnen benoetigt
        clr     r15             ;und mit Null belegt
.DEF	Tmp1 = r16		;Register 16 dient als erster Zwischenspeicher
.DEF	Tmp2 = r17		;Register 17 dient als zweiter Zwischenspeicher
.DEF    Cnt1 = r18              ;globale Zaehlvariable
.DEF    WL = r24                ;Register 24 und 25 dienen als universelles
.DEF    WH = r25                ;Doppelregister W

	;Stapel initialisieren (fuer Unterprogramme bzw. Interrupts)
	ldi	Tmp1,HIGH(RAMEND)	;RAMEND (SRAM) ist in der Definitions-
	out	SPH,Tmp1		;datei festgelegt
	ldi	Tmp1,LOW(RAMEND)
	out	SPL,Tmp1

        ldi     r16,0x80        ;CLKPCE = 1 Wirkung des Fusebit CKDIV
        sts     CLKPR,r16       ;ueberschreiben
        clr     r16             ;CLKPR = 0
        sts     CLKPR,r16

        ;3 Sekunden warten bis VDrive startbereit
        clr     XH              ;
        ldi     XL,3            ;
        rcall   WX1s            ;

        ;PD6 als Ausgang
        sbi     DDRD,6          ;Teensy LED!
        sbi     DDRD,7          ;Summer!

        ;Zeitschleifendauer initialisieren (in ms)
        ldi     XL,LOW(2000)
        ldi     XH,HIGH(2000)
        
        rcall   I2CINI
        rcall	LCDINI
        rcall   LCDCLR

;-------------------------------------------------------------------------------
;	Hauptprogramm
;-------------------------------------------------------------------------------
MAIN:   ;Ausgabe LCD
        ldi     r24,0x80		;1 Zeile
        rcall   LCDSCO
        ldi     ZL,LOW(TEXT1*2)
	ldi     ZH,HIGH(TEXT1*2)
        rcall   LCDSTF          
        ldi     r24,0xC6        ;2 Zeile
        rcall   LCDSCO
        ldi     r24,'0'	
        rcall   LCDOUT
        ldi     r24,'x'	
        rcall   LCDOUT
	rcall   CO2REP		;Lese Sensoradresse aus dem EEPROM
	mov	Tmp1,r24
	rcall	HEX2A2		;Umwandlung nach ASCII
	mov	Tmp2,r24
        mov	r24,r25
	rcall   LCDOUT		;Ausgabe	
        mov	r24,Tmp2
        rcall   LCDOUT
        rcall	W1s
        rcall	W1s
        rcall	W1s
        cpi	Tmp1,0x11	;Falls nicht 0x11 dann Aendern
	breq	END
        
CHANGE: ;Ausgabe LCD
        ldi     r24,0x80		;1 Zeile
        rcall   LCDSCO
        ldi     ZL,LOW(TEXT2*2)
	ldi     ZH,HIGH(TEXT2*2)
        rcall   LCDSTF          
        ldi     r24,0xC0		;2 Zeile
        rcall   LCDSCO
        ldi     ZL,LOW(TEXT3*2)
	ldi     ZH,HIGH(TEXT3*2)
        rcall   LCDSTF          
        rcall	W10s		;Warte 10s bevor geaendert wird

	rcall	CO2SEP	 	;Aendere Sensoradresse auf 0x11

	rcall   LCDCLR
        ldi     r24,0x80		;1 Zeile
        rcall   LCDSCO
        ldi     ZL,LOW(TEXT4*2)
	ldi     ZH,HIGH(TEXT4*2)
        rcall   LCDSTF
        ldi     r24,0xC0		;2 Zeile
        rcall   LCDSCO
        ldi     ZL,LOW(TEXT5*2)
	ldi     ZH,HIGH(TEXT5*2)
        rcall   LCDSTF


END:    rjmp    END
 
;-------------------------------------------------------------------------------
;	Unterprogramme und Interrupt-Behandlungsroutinen
;-------------------------------------------------------------------------------

.INCLUDE "SR_TIME_16M.asm"  ;Zeitschleifen Bibliothek
.INCLUDE "SR_I2C.asm"      ;I2C Bibliothek
.INCLUDE "SR_CO2.asm"           ;CO2 Bibliothek
.INCLUDE "SR_MUL_DIV.asm"       ;Rechen Bibliothek
.INCLUDE "SR_RECODE.asm"        ;Umkodier Bibliothek
.INCLUDE "SR_LCD_2x16_4Bit.asm"     ;LCD Bibliothek

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EXIT				;Ende des Quelltextes
